#include <stdlib.h>
#include <error.h>
#include <errno.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include <linux/types.h>
#include <sys/capability.h>
#include <sys/prctl.h>

#include "dropcap.h"

extern void drop_cap (void);

void
drop_cap (void)
{
	const char *dir = "/var/resolv";
	cap_t   caps;
	uid_t uid = getuid ();
	gid_t gid = getgid ();

	if (uid && geteuid())
		return;

	if (setgroups (0, NULL) < 0)
		error (EXIT_FAILURE, errno, "setgroups");

	if (prctl (PR_SET_KEEPCAPS, 1))
		error (EXIT_FAILURE, errno, "prctl (PR_SET_KEEPCAPS, 1) failed");

	if (!uid)
	{
		const char *user = "iputils";
		struct passwd *pw = getpwnam (user);

		if (!pw)
			error (EXIT_FAILURE, 0,
			       "lookup of user \"%s\" failed", user);

		gid = pw->pw_gid;
		uid = pw->pw_uid;
		endpwent ();

		if (!uid)
			error (EXIT_FAILURE, 0,
			       "user \"%s\" shouldn't be root", user);
	}

	if (chroot (dir) < 0)
		error (EXIT_FAILURE, errno, "chroot to \"%s\" failed", dir);

	if (chdir ("/") < 0)
		error (EXIT_FAILURE, errno, "chdir to \"/\" failed");

	if (setgid (gid) < 0)
		error (EXIT_FAILURE, errno, "setgid");

	caps = cap_from_text ("cap_setuid,cap_net_raw=ep");
	if (!caps)
		error (EXIT_FAILURE, errno, "cap_from_text failed");

	if (cap_set_proc (caps) < 0)
		error (EXIT_FAILURE, errno, "cap_set_proc failed");

	cap_free (caps);

	if (setreuid (uid, uid) < 0)
		error (EXIT_FAILURE, errno, "setreuid");

	caps = cap_from_text ("cap_net_raw=ep");
	if (!caps)
		error (EXIT_FAILURE, errno, "cap_from_text failed");

	if (cap_set_proc (caps) < 0)
		error (EXIT_FAILURE, errno, "cap_set_proc failed");

	cap_free (caps);
}
