#include <stdlib.h>
#include <error.h>
#include <errno.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>

#include "droppriv.h"

uid_t   uid = -1;

void
drop_priv (void)
{
	const char *dir = "/var/resolv";
	uid = getuid ();

	if (uid && geteuid())
		return;

	if (setgroups (0, NULL) < 0)
		error (EXIT_FAILURE, errno, "setgroups");

	if (uid)
	{
		if (chroot (dir) < 0)
			error (EXIT_FAILURE, errno, "chroot to \"%s\" failed",
			       dir);

		if (chdir ("/") < 0)
			error (EXIT_FAILURE, errno, "chdir to \"/\" failed");

		if (setgid (getgid ()) < 0)
			error (EXIT_FAILURE, errno, "setgid");

		if (setuid (uid) < 0)
			error (EXIT_FAILURE, errno, "setuid");
	} else
	{
		const char *user = "iputils";
		struct passwd *pw = getpwnam (user);

		if (!pw)
			error (EXIT_FAILURE, 0,
			       "lookup of user \"%s\" failed", user);
		endpwent ();

		if (!pw->pw_uid)
			error (EXIT_FAILURE, 0,
			       "user \"%s\" shouldn't be root", user);

		if (chroot (dir) < 0)
			error (EXIT_FAILURE, errno, "chroot to \"%s\" failed",
			       dir);

		if (chdir ("/") < 0)
			error (EXIT_FAILURE, errno, "chdir to \"/\" failed");

		if (setgid (pw->pw_gid) < 0)
			error (EXIT_FAILURE, errno, "setgid");

		if (setuid (pw->pw_uid) < 0)
			error (EXIT_FAILURE, errno, "setuid");
	}
}
